import * as THREE from "three";
import { useEffect, useMemo, useRef, useState } from "react";
import { useFrame, useLoader } from "@react-three/fiber";
import { DRACOLoader } from "three/examples/jsm/loaders/DRACOLoader";
import { GLTFLoader } from "three/examples/jsm/loaders/GLTFLoader";
import { clone } from "three/examples/jsm/utils/SkeletonUtils";
import { CCDIKSolver, CCDIKHelper, } from "three/examples/jsm/animation/CCDIKSolver";
import IKAnimationController from "./IKAnimationController";
import { TransformControls } from "@react-three/drei";

const IkInstances = ({
    uuid,
    modelUrl,
    processes,
    position,
    rotation,
    updateArmBotStatus
}: {
    uuid: string;
    modelUrl: string;
    processes: any;
    position: [number, number, number];
    rotation: [number, number, number];
    updateArmBotStatus: (status: string) => void;
}) => {
    const [ikSolver, setIkSolver] = useState<any>(null);
    const [selectedTrigger, setSelectedTrigger] = useState<string>("");
    const gltf = useLoader(GLTFLoader, modelUrl, (loader) => {
        const draco = new DRACOLoader();
        draco.setDecoderPath("https://cdn.jsdelivr.net/npm/three@0.160.0/examples/jsm/libs/draco/");
        loader.setDRACOLoader(draco);
    });
    const cloned = useMemo(() => clone(gltf.scene), [gltf]);
    const groupRef = useRef<any>(null);
    const targetBoneName = "Target";
    const skinnedMeshName = "link_0";

    useEffect(() => {
        if (!gltf) return;
        const OOI: any = {};
        cloned.traverse((n: any) => {
            if (n.name === targetBoneName) OOI.Target_Bone = n;
            if (n.name === skinnedMeshName) OOI.Skinned_Mesh = n;
        });

        if (!OOI.Target_Bone || !OOI.Skinned_Mesh) return;

        const iks = [
            {
                target: 7,
                effector: 6,
                links: [
                    {
                        index: 5,
                        enabled: true,
                        rotationMin: new THREE.Vector3(-Math.PI / 2, 0, 0),
                        rotationMax: new THREE.Vector3(Math.PI / 2, 0, 0),
                    },
                    {
                        index: 4,
                        enabled: true,
                        rotationMin: new THREE.Vector3(-Math.PI / 2, 0, 0),
                        rotationMax: new THREE.Vector3(0, 0, 0),
                    },
                    {
                        index: 3,
                        enabled: true,
                        rotationMin: new THREE.Vector3(0, 0, 0),
                        rotationMax: new THREE.Vector3(2, 0, 0),
                    },
                    { index: 1, enabled: true, limitation: new THREE.Vector3(0, 1, 0) },
                    { index: 0, enabled: false, limitation: new THREE.Vector3(0, 0, 0) },
                ],
            },
        ];

        const solver = new CCDIKSolver(OOI.Skinned_Mesh, iks);
        setIkSolver(solver);

        const helper = new CCDIKHelper(OOI.Skinned_Mesh, iks, 0.05);
        // groupRef.current.add(helper);

    }, [gltf]);

    useEffect(() => {
        const triggers = ["9f4a9b8b-e60d-4754-8c99-d71979da0e71", "b77b4f0a-ce55-4fe0-a181-a43ab3d01c83"];
        let index = 0;

        const cycleTriggers = setInterval(() => {
            setSelectedTrigger(triggers[index]);
            index = (index + 1) % triggers.length;
        }, 10000);

        return () => clearInterval(cycleTriggers);
    }, []);

    const logStatus = (status: string) => {
        // console.log(status);
    }

    return (
        <>
            <group
                ref={groupRef}
                position={position}
                rotation={rotation}
            >
                <primitive
                    object={cloned}
                    scale={[1, 1, 1]}
                    name={`arm-bot`}
                />
            </group>
            <IKAnimationController
                ikSolver={ikSolver}
                process={processes}
                selectedTrigger={selectedTrigger}
                targetBoneName={targetBoneName}
                uuid={uuid}
                logStatus={logStatus}
                groupRef={groupRef}
                updateArmBotStatus={updateArmBotStatus}
            />
        </>
    );
};

export default IkInstances;